#//////////////////////////////////////////////////////////#
#///R Script                                            ///#
#///Political Conflict in the Reform of the Eurozone    ///#
#///Fabio Wasserfallen & Thomas Lehner - 2018           ///#
#///macOS High Sierra Version 10.13.4                   ///#
#///Software: R (Version 3.5.0)                         ///#
#///Software: R-Studio (Version 1.1.453)                ///#
#///Last Updated: 2018-10-16                             ///#
#//////////////////////////////////////////////////////////#


##############################################################################
##############################################################################
#                  Clear Global Environment and load packages                #
##############################################################################
##############################################################################

rm(list=ls()) 

# Load Packages (Note: This command will install packages that have not yet been installed and loads
## them automatically thereafter (this may be particularly relevant for the basicspace package)

library(wnominate)  #https://cran.r-project.org/web/packages/wnominate/index.html
library(basicspace) #https://cran.r-project.org/web/packages/basicspace/index.html
library(MCMCpack)   #https://cran.r-project.org/web/packages/MCMCpack/index.html


#############################################################################
#############################################################################
#                 Prep. 1: Set Working Directory and read in data           #
#############################################################################
#############################################################################

setwd("/.../Replication Archivee")
## Read in Dataset 
DataM<-read.csv("ScoresFR.csv", header=T, sep=",")

#MS as rownames
rownames(DataM)<-DataM[,1]
DataM<-DataM[,-1]

#remove OUT (dataset with original data for IRT model)
Dat<-DataM[-35,]

#create binary data for W-Nominate Model
for (i in  c(1:35)){
  for (j in c(1:47)){
    
    if (is.na(DataM[i,j])){DataM[i,j]<-NA}
    else if (DataM[i,j]<49) {DataM[i,j]<-0}
    else if (DataM[i,j]>49) {DataM[i,j]<-1}
  }}
Datwn<-DataM[-35,]
rm(DataM)

#############
# 1.        #
# Run Models#
#           #
#############

############################################################
############################################################
###                Run W-Nominate analysis               ###
############################################################
############################################################

#prepare RollCall
rc<-rollcall(Datwn[,1:ncol(Datwn)],
             yea=1,
             nay=0,
             missing=c(NA),
             notInLegis=5,
             legis.names=rownames(Datwn),
             vote.names=colnames(Datwn[1:ncol(Datwn)]),
             legis.data=NULL,
             vote.data=NULL,
             desc="Nominate for CURR data")
rc$legis.data$Vote.ID

result<-wnominate(rc, ubeta=15, uweight=0.5, dims=2, minvotes=10,
                  lop=0.025, trials=3, polarity=c(6,13), verbose=FALSE)
summary(result) 

###############################################
###############################################
## Run Main Analysis - One dimensional IRT ####
###############################################
###############################################
x<-as.matrix(Dat)
posterior<-MCMCordfactanal(x,factors=1, lambda.constraints=list(FC6=list(2,"+")),
                           data=Dat,
                           burnin = 1000, mcmc = 20000,
                           thin=100, tune=NA, verbose = FALSE, seed = 123,
                           lambda.start = NA, l0=0, L0=0,
                           store.lambda=TRUE, store.scores=TRUE,
                           drop.constantvars=TRUE)
means.sds<-summary(posterior)[[1]][,1:2]
ideal.points<-means.sds[grepl("phi", rownames(means.sds)),]
item.params<-means.sds[grepl("Lambda", rownames(means.sds)),]



########################################
########################################
#### Plots and Table 1 for main text ###
########################################
########################################


############################################################
#              plot Figure 4 - ScreePlot                   #
############################################################

plot.scree(result, main.title = "")
eigenvalues<-result$eigenvalues[c(1:20)]
plot(eigenvalues, type="l", ylab="Eigenvalue", xlab = "Dimension", yaxt="n", xaxt="n",cex=1, cex.lab=1.4)
points(eigenvalues, pch=16)
axis(side=1,at=seq(from=1,to=19,by=2), cex.axis=1.3)
axis(side=2,at=1, cex.axis=1.3)
#Export the graph as png with the following proportions: Width:1270, Height:780

########################################
#  plot Figure 5 - One dimensional IRT #
########################################

irt1<-as.data.frame(ideal.points[,1])
rownames(irt1)<-rownames(Dat)
irt1<-cbind(irt1,c(1:34))
colnames(irt1)<-c("idealpoint", "pos")
irt1[1,2]<-0.9 #position Austria
irt1[2,2]<-0 #position BEL
irt1[3,2]<-0.6 #position BGR
irt1[4,2]<-0 #position CYP
irt1[5,2]<-0.6 #position CZE
irt1[6,2]<-0 #position DEU
irt1[7,2]<-1.2 #position DNK
irt1[8,2]<-0 #position ESP
irt1[9,2]<-0.3 #position EST
irt1[10,2]<-0 #position FIN
irt1[11,2]<-0 #position FRA
irt1[12,2]<-1.8 #position GBR
irt1[13,2]<-0 #position GRC
irt1[14,2]<-0.3 #position HRV
irt1[15,2]<-0 #position HUN
irt1[16,2]<-0 #position IRL
irt1[17,2]<-0.3 #position ITA
irt1[18,2]<-0.6 #position LTU
irt1[19,2]<-0.9 #position LUX
irt1[20,2]<-1.2 #position LVA
irt1[21,2]<-1.5 #position MLT
irt1[22,2]<-0 #position NLD
irt1[23,2]<-0.3 #position POL
irt1[24,2]<-0 #position PRT
irt1[25,2]<-1.5 #position ROU
irt1[26,2]<-0 #position SVK
irt1[27,2]<-0.3 #position SVN
irt1[28,2]<-0.9 #position SWE
irt1[29,2]<-0.3 #position COM
irt1[30,2]<-0 #position COU
irt1[31,2]<-0.6 #position ECB
irt1[32,2]<-0 #position EFN
irt1[33,2]<-0.6 #position EP
irt1[34,2]<-0.3 #position EUG


plot(irt1$idealpoint,irt1$pos,main = "",xlab="Fiscal transfer                                                                                                                                    Fiscal discipline",
     cex.lab=1.2, ylab="",xlim=c(-0.145,0.225),ylim=c(-0.2,2.6), type="n",frame.plot = F,axes=F, yaxt="n")

text(irt1$idealpoint, irt1$pos,rownames(irt1) ,cex=1)
axis(side=1, labels=F, at=c(-0.145,-0.0525,0.04,0.1325,0.225))
#Export the graph as png with the following proportions: Width:1180, Height:320

############################################################
#   Calculate cutting line angles for Table 1              #
############################################################
DL1<-result$rollcalls[,7]
DL2<-result$rollcalls[,8]
ZM1<-result$rollcalls[,9]
ZM2<-result$rollcalls[,10]
Yes1<-ZM1-DL1
WEIGHT<-(result$weights[2])/(result$weights[1])
Yes2w<-(ZM2 - DL2)*WEIGHT
No1<-ZM1+DL1
No2w<-(ZM2+DL2)*WEIGHT
A1<- No1-Yes1
A2<-No2w-Yes2w
ALENGTH<-sqrt(A1*A1+A2*A2)
N1W<-A1/ALENGTH
N2W<-A2/ALENGTH
for (i in 1:nrow(result$rollcalls)){
  if (N1W[i]<0 & !is.na(N2W[i])) N2W[i]<- -N2W[i]
  if (N1W[i]<0 & !is.na(N1W[i])) N1W[i]<- - N1W[i]
}
##calculate cutting points
ws<-N1W*ZM1+N2W*ZM2*WEIGHT
xws<-ws*N1W
yws<-ws*N2W
##calulate cutting line angels
C1<-N2W
C2<- -N1W
for (i in 1:nrow(result$rollcalls)){
  if (C1[i]<0 & !is.na(C2[i])) C2[i]<- -C2[i]
  if (C1[i]<0 & !is.na(C1[i])) C1[i]<- -C1[i]
}
theta<-atan2(C2, C1)
theta4<-theta*(180/pi)

#store cutting angles in dataframe
#Theata4=cutting line angles in Table 1
cut<-cbind (theta4, result$rollcalls)
rownames(cut)<-colnames(Dat)

rm(A1,A2, ALENGTH, C1,C2,DL1,DL2,i,j,N1W, N2W, No1, No2w,theta,theta4,WEIGHT,ws,xws,Yes1,Yes2w,yws,ZM1,ZM2)
write.csv(cut, "cutangelsM.csv", na="")



########################################
########################################
######## Plots for the Appendix ########
########################################
########################################



##############################################################
###         Basic Space Scaling - 1 Dimensional            ###
##############################################################
#transform into numeric
for (i in c(1:47)){
  Dat[,i]<-as.numeric(Dat[,i])
}

issues<-as.matrix(Dat)

result<-blackbox(issues, dims=1, minscale=10, verbose=T)

xx<-result$individuals[[1]][,1]
#############################################################
#  plot Figure 1 - Appendix 1 - One dimensional Basic Space #
#############################################################

B1<-cbind(rownames(Dat),xx )
B1pos<-c(0.3,0.3,0.6,0,1.5,1.2,0.6,0.9,0,0.3,0.3,0,0.6,0.9,0.3,0.6,1.2,0,0,0.9,0.6,0,0.3,0,0.9,0.6,0.3,1.2,0.9,0,0.3,0,0.6,0.3)
B1<-as.data.frame(B1)
B1$pos<- B1pos
B1$xx<-as.numeric(as.character(B1$xx))
plot(xx, B1pos, main = "",xlab="Fiscal transfer                                                                                                          Fiscal discipline",cex.lab=1.2, ylab="",xlim=c(-10,10),ylim=c(-0.1,1.7), type="n",frame.plot = F,axes=FALSE, yaxt="n")
axis(side=1, labels=F, at=c(8.6,4.3,0,-4.3,-8.6))
text(B1$xx,B1$pos,B1$V1, cex=1)
#Export the graph as png with the following proportions: Width:1180, Height:320

############################################################
###          Run 1 Dimensional W-Nominate                ###
############################################################
result<-wnominate(rc, ubeta=15, uweight=0.5, dims=1, minvotes=10,
                  lop=0.025, trials=3, polarity=6, verbose=FALSE)#6=germany to set polarity

country<-as.character(rownames(Dat))
X<-result$legislators$coord1D

##############################################################
#  plot Figure 2 - Appendix 2 - One dimensional W-Nominate #
##############################################################
WNOM<-cbind(country, X) 
WNOM<-as.data.frame(WNOM)
str(WNOM)
for (i in 2:ncol(WNOM)){
  WNOM[,i] <- as.numeric(as.character(WNOM[,i] ))
}    
WNOM$pos<-c(1.2,0.3,0.6,0,1.5,0.9,0.9,0.3,0.3,0.6,0,0,0,0.3,0,0.3,0.6,0,0,0.9,0.6,1.2,0.3,0,0.3,0.3,0.3,1.2,0.6,0.9,0.3,0,0.6,0)
plot(WNOM$X, WNOM$pos, main = "",xlab="Fiscal transfer                                                                                   Fiscal discipline",cex.lab=1.2, ylab="",xlim=c(-1.2,1.2),ylim=c(-0.2,1.7), type="n",frame.plot = F,axes=FALSE, yaxt="n")
axis(side=1, labels=F)
text(WNOM$X,WNOM$pos,WNOM$country, cex=1)
#Export the graph as png with the following proportions: Width:1180, Height:320

##########################
## IRT- including OUT ####
##########################
DataM<-read.csv("ScoresFR.csv", header=T, sep=",")

#MS as rownames
rownames(DataM)<-DataM[,1]
DataM<-DataM[,-1]

### Ordinal IRT Model #############
x<-as.matrix(DataM)
posterior<-MCMCordfactanal(x,factors=1, lambda.constraints=list(FC6=list(2,"+")),
                           data=DataM,
                           burnin = 1000, mcmc = 20000,
                           thin=100, tune=NA, verbose = FALSE, seed = 123,
                           lambda.start = NA, l0=0, L0=0,
                           store.lambda=TRUE, store.scores=TRUE,
                           drop.constantvars=TRUE)
means.sds<-summary(posterior)[[1]][,1:2]
ideal.points<-means.sds[grepl("phi", rownames(means.sds)),]
item.params<-means.sds[grepl("Lambda", rownames(means.sds)),]

################################################################
#  plot Figure 3 - Appendix 3 -  One dimensional IRT with OUT  #
################################################################
irt1<-as.data.frame(ideal.points[,1])
rownames(irt1)<-rownames(DataM)
irt1<-cbind(irt1,c(1:35))
colnames(irt1)<-c("idealpoint", "pos")
irt1[1,2]<-0.9 #position Austria
irt1[2,2]<-0.6 #position BEL
irt1[3,2]<-2.8 #position BGR
irt1[4,2]<-0.6 #position CYP
irt1[5,2]<-0 #position CZE
irt1[6,2]<-0 #position DEU
irt1[7,2]<-2.1 #position DNK
irt1[8,2]<-0.3 #position ESP
irt1[9,2]<-0.3 #position EST
irt1[10,2]<-0 #position FIN
irt1[11,2]<-0 #position FRA
irt1[12,2]<-1.5 #position GBR
irt1[13,2]<-0 #position GRC
irt1[14,2]<-0.3 #position HRV
irt1[15,2]<-0.9 #position HUN
irt1[16,2]<-0 #position IRL
irt1[17,2]<-0 #position ITA
irt1[18,2]<-1.8 #position LTU
irt1[19,2]<-0.9 #position LUX
irt1[20,2]<-0.6 #position LVA
irt1[21,2]<-0 #position MLT
irt1[22,2]<-0 #position NLD
irt1[23,2]<-0.6 #position POL
irt1[24,2]<-0 #position PRT
irt1[25,2]<-2.4 #position ROU
irt1[26,2]<-1.5 #position SVK
irt1[27,2]<-0 #position SVN
irt1[28,2]<-1.2 #position SWE
irt1[29,2]<-0.3 #position COM
irt1[30,2]<-0 #position COU
irt1[31,2]<-1.2 #position ECB
irt1[32,2]<-0 #position EFN
irt1[33,2]<-0.3 #position EP
irt1[34,2]<-0.3 #position EUG
irt1[35,2]<-0.6 #position OUT

plot(irt1$idealpoint,irt1$pos,main = "",xlab="Fiscal transfer                                                                                                                                    Fiscal discipline",
     cex.lab=1.2, ylab="",xlim=c(-0.43,0.01),ylim=c(-0.2,2.9), type="n",frame.plot = F,axes=F, yaxt="n")

text(irt1$idealpoint, irt1$pos,rownames(irt1) ,cex=1)
axis(side=1, labels=F, at=c(-0.43,-0.32,-0.21,-0.1,0.01))
#Export the graph as png with the following proportions: Width:1180, Height:320

############################################################
# plot Figure 4 - Appendix 4 - Two-Dimensional W-Nominate  #             
############################################################
result<-wnominate(rc, ubeta=15, uweight=0.5, dims=2, minvotes=10,
                  lop=0.025, trials=3, polarity=c(6,13), verbose=FALSE)
summary(result) 

country<-as.character(rownames(Datwn))
X<-result$legislators$coord1D
Y<-result$legislators$coord2D
WNOM<-cbind(country, X, Y)
WNOM<-as.data.frame(WNOM)
str(WNOM)
for (i in 2:ncol(WNOM)){
  WNOM[,i] <- as.numeric(as.character(WNOM[,i] ))
}
plot(WNOM$X, WNOM$Y, main="",xlab= "First dimension", ylab="Second dimension",xlim=c(-1,1), ylim=c(-1,1), type="n", cex.axis=1.3, cex.lab=1.3)
text(WNOM$X, WNOM$Y, WNOM$country, cex=0.9)
#Export the graph as png with the following proportions: Width:690, Height:590

####################################################################################
# plot Figure 5 - Annex 6.5.Two-Dimensional W-Nominate with cutting line of FC3   #             
####################################################################################

nrollcall<-24
vote<-as.integer(rc$votes[,nrollcall])

DL1<-result$rollcalls[nrollcall,7]
DL2<-result$rollcalls[nrollcall,8]
ZM1<-result$rollcalls[nrollcall,9]
ZM2<-result$rollcalls[nrollcall,10]

WEIGHT<-(result$weights[2])/(result$weights[1])
Yea1<-ZM1-DL1*WEIGHT
Yea2W<-(ZM2-DL2)*WEIGHT
NAY1<-ZM1+DL1
NAY2W<-(ZM2+DL2)*WEIGHT

A1<-NAY1-Yea1
A2<-NAY2W-Yea2W

ALENGTH<-sqrt(A1*A1+A2*A2)
N1W<-A1/ALENGTH
N2W<-A2/ALENGTH
if(N1W<0){
  N1W<- -N1W
  N2W<- -N2W
}

ws<-N1W*ZM1 + N2W*ZM2*WEIGHT
xws<-ws*N1W
yws<-ws*N2W

#plot cutting line for FC3
segments(xws+N2W, yws-N1W, xws-N2W, yws+N1W, lw=2, col="black")
#Export the graph as png with the following proportions: Width:690, Height:590
